/**
 * An object that specifies characteristics of a sorted container.
 *
 * @typeParam T - The type of comparable elements.
 */
interface SortedArrayConstructorOptions<T> {
    /**
     * Function used to determine the order of the elements.
     * It should return a negative number if the first argument is less than the second argument,
     * a positive number if the first argument is greater than the second argument, or a zero otherwise.
     * The comparator should always return a number, and never return NaN.
     * In addition, the comparator has the same requirements as the parameter of `Array.prototype.sort`
     * (purity, stability, reflexivity, anti-symmetry, and transitivity).
     *
     * If omitted, the elements are compared with operators `<`, `===`, and `>`.
     * These operators violate transitivity if the inputs have mixed types or include NaN.
     * See [Custom Comparators](../docs/comparator.md) for more information.
     */
    comparator?: (a: T, b: T) => number;
    /**
     * Specifies the load-factor for sorted container sublists.
     * The default load factor of 1000 works well for arrays from tens to tens-of-millions of values.
     * Good practice is to use a value that is the cube root of the list size.
     * With billions of elements, the best load factor depends on your usage.
     * It's best to leave the load factor at the default until you start benchmarking.
     *
     * @see {@link https://grantjenks.com/docs/sortedcontainers/implementation.html | Implementation Details} in Python Sorted Containers
     * @see {@link https://grantjenks.com/docs/sortedcontainers/performance-scale.html | Performance at Scale} in Python Sorted Containers
     */
    loadFactor?: number;
}
/**
 * A very leaky abstraction (if it can be called an abstraction at all) just meant to share code,
 * because it is painful to get decent method delegation in JavaScript.
 *
 * You should not be using it directly.
 */
declare abstract class AbstractSortedArray<T extends C, C> {
    /**
     * The default load factor.
     * Override it with option `loadFactor` at construction time.
     */
    static DEFAULT_LOAD_FACTOR: number;
    /**
     * Initialize a sorted container instance.
     *
     * Optional `iterable` argument provides an initial iterable of values to initialize the sorted container.
     *
     * @param iterable - Initial values (optional).
     * @param options - An object that specifies characteristics about the sorted container.
     */
    constructor(iterable?: Iterable<T>, options?: SortedArrayConstructorOptions<C>);
    /**
     * Remove all values from sorted container.
     */
    clear(): void;
    /**
     * Update sorted container by adding all values from `iterable`.
     *
     * @example
     * const sl = new SortedArray();
     * sl.update([3, 1, 2]);
     * sl // SortedArray [1, 2, 3]
     *
     * @param iterable - Iterable of values to add.
     */
    abstract update(iterable: Iterable<any>): void;
    /**
     * Remove one `value` from sorted container if it is a member.
     *
     * If `value` is not a member, do nothing.
     *
     * @example
     * const sl = new SortedArray([1, 2, 3, 4, 5])
     * sl.delete(5)
     * sl.delete(0)
     * Array.from(sl) // [1, 2, 3, 4]
     * @param value - Value to discard from SortedArray.
     * @returns Returns true if an element in the SortedArray existed and has been removed, or false if the element does not exist.
     */
    delete(value: C): boolean;
    /**
     * Remove and return value at `index` in sorted container.
     *
     * If the sorted container is empty or index is out of range, `undefined` is returned and the sorted container is not modified.
     *
     * Negative indices count back from the last item.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.deleteAt(-1) // 'e'
     * sl.deleteAt(2) // 'c'
     * sl // SortedArray ['a', 'b', 'd']
     *
     * @param index - Index of value. Negative integers count back from the last item in the array.
     * @returns The removed value or `undefined`.
     */
    deleteAt(index: number): T | undefined;
    /**
     * Removes elements from sorted container by index.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.deleteSlice(0, 2);
     * sl // SortedArray ['c', 'd', 'e']
     *
     * @param start The zero-based location in the array from which to start removing elements.
     * Negative integers count back from the last item in the array.
     * @param end The zero-based location in the array at which to stop removing elements.
     * Negative integers count back from the last item in the array.
     * Elements up to but not including `end` are removed.
     */
    deleteSlice(start?: number, end?: number): void;
    /**
     * Returns a copy of a section of sorted container as an ordinary Array.
     *
     * For both start and end, a negative index can be used to indicate an offset from the end of the array.
     *
     * @param start The beginning index of the specified portion of the array.
     * If start is undefined, then the slice begins at index 0.
     * @param end The end index of the specified portion of the array. This is exclusive of the element at the index `end`.
     * If end is undefined, then the slice extends to the end of the array.
     */
    slice(start?: number, end?: number): T[];
    /**
     * Lookup value at `index` in sorted container.
     *
     * If `index` is out of range, returns `undefined`.
     *
     * @example
     * const sl = new SortedList('abcde');
     * sl.at(1) // 'b'
     * sl.at(-1) // 'e'
     * sl.at(7) // undefined
     *
     * @param index - The zero-based index of the desired code unit. A negative index will count back from the last item.
     * @returns The item located at the specified index, or `undefined` if index out of range.
     */
    at(index: number): T | undefined;
    /**
     * Return an iterator over the sorted container.
     *
     * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     */
    _iter(): IteratorObject<T, undefined, unknown>;
    /**
     * Return a reverse iterator over the sorted container.
     *
     * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     */
    reversed(): IteratorObject<T, undefined, unknown>;
    /**
     * Return an iterator that slices sorted container from `start` to `end`.
     *
     * The `start` and `end` index are treated inclusive and exclusive, respectively.
     *
     * A negative index will count back from the last item.
     *
     * Both `start` and `end` default to `undefined` which is automatically inclusive of the beginning and end of the sorted container.
     *
     * When `reverse` is `true` the values are yielded from the iterator in reverse order; `reverse` defaults to `false`.
     *
     * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     *
     * @example
     * const sl = new SortedArray('abcdefghij');
     * const it = sl.islice(2, 6);
     * Array.from(it) // ['c', 'd', 'e', 'f']
     *
     * @param start - Start index (inclusive).
     * @param end - Stop index (exclusive).
     * @param reverse - Whether to yield values in reverse order.
     * @returns Iterator.
     */
    islice(start?: number, end?: number, reverse?: boolean): IteratorObject<T, undefined, unknown>;
    /**
     * Create an iterator of values between `minimum` and `maximum`.
     *
     * Both `minimum` and `maximum` default to `undefined` which is automatically inclusive of the beginning and end of the sorted container.
     *
     * The argument `includeMinimum` and `includeMaximum` is a pair of booleans that indicates whether the minimum and maximum ought to be included in the range, respectively.
     * Both arguments default to `true` such that the range is inclusive of both minimum and maximum.
     *
     * When `reverse` is `true` the values are yielded from the iterator in reverse order; `reverse` defaults to `false`.
     *
     * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     *
     * @example
     * const sl = new SortedArray('abcdefghij');
     * const it = sl.irange('c', 'f');
     * Array.from(it) // ['c', 'd', 'e', 'f']
     *
     * @param minimum - Minimum value to start iterating.
     * @param maximum - Maximum value to stop iterating.
     * @param includeMinimum - Whether the minimum ought to be included in the range.
     * @param includeMaximum - Whether the maximum ought to be included in the range.
     * @param reverse - Whether to yield values in reverse order.
     * @returns Iterator.
     */
    irange(minimum?: C, maximum?: C, includeMinimum?: boolean, includeMaximum?: boolean, reverse?: boolean): IteratorObject<T, undefined, unknown>;
    /**
     * Return an index to insert `value` in the sorted container.
     *
     * If the `value` is already present, the insertion point will be before (to the left of) any existing values.
     *
     * @example
     * const sl = new SortedArray([10, 11, 12, 13, 14]);
     * sl.bisectLeft(12) // 2
     *
     * @param value - Value to find the insertion point of.
     * @returns Insertion index of `value` in the sorted container.
     */
    bisectLeft(value: C): number;
    /**
     * Return an index to insert `value` in the sorted container.
     *
     * Similar to {@link bisectLeft}, but if `value` is already present, the insertion point will be after (to the right of) any existing values.
     *
     * @example
     * const sl = new SortedArray([10, 11, 12, 13, 14]);
     * sl.bisectRight(12) // 3
     *
     * @param value - Value to find the insertion point of.
     * @returns Insertion index of `value` in the sorted container.
     */
    bisectRight(value: C): number;
    /**
     * Return a shallow copy of the sorted container.
     *
     * @returns A new sorted container.
     */
    clone(): this;
    /**
     * Remove the first element from the sorted container and return it.
     * If the sorted container is empty, undefined is returned and the sorted container is not modified.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.shift() // 'a'
     * sl.shift() // 'b'
     * sl // SortedArray ['c', 'd', 'e']
     *
     * @returns Shifted value or undefined.
     */
    shift(): T | undefined;
    /**
     * Remove the last element from the sorted container and return it.
     * If the sorted container is empty, undefined is returned and the sorted container is not modified.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.pop() // 'e'
     * sl.pop() // 'd'
     * sl // SortedArray ['a', 'b', 'c']
     *
     * @returns Popped value or undefined.
     */
    pop(): T | undefined;
    /**
     * Return first index of `value` in the sorted container, or -1 if `value` is not present.
     *
     * Index must be between `start` and `end` for the `value` to be considered present.
     * Negative indices count back from the last item.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.indexOf('d') // 3
     * sl.indexOf('z') // -1
     *
     * @param value - Value in sorted container.
     * @param start - The array index at which to start the search. If omitted, defaults to 0.
     * @param end - The array index at which to end the search. If omitted, defaults to the end of the sorted container.
     * @returns The index of the first occurrence of `value` in the sorted container, or -1 if it is not present.
     */
    indexOf(value: C, start?: number, end?: number): number;
    /**
     * Return the first element in the sorted container that is equal to the provided value.
     * If it is not found, undefined is returned.
     *
     * With the default comparator and primitive elements,
     * this method does not tell anything beyond whether the value exists in the sorted container.
     * However, it can turn out to be useful with a custom comparator.
     *
     * @example
     * const sl = new SortedArray(
     *   [
     *     { id: 1, value: 'one' },
     *     { id: 2, value: 'two' },
     *     { id: 3, value: 'three' },
     *   ],
     *   { comparator: (a, b) => a.id - b.id }
     * );
     * sl.find({ id: 2 }) // { id: 2, value: 'two' }
     *
     * @param value - The value to find.
     */
    find(value: C): T | undefined;
    /**
     * Return sorted container as an ordinary Array.
     */
    toJSON(): T[];
    /**
     * Return a string representation of sorted container.
     */
    toString(): string;
}

/**
 * SortedArray is a sorted mutable collection.
 *
 * SortedArray values are maintained in sorted order.
 *
 * SortedArray values must have a total ordering.
 * The total ordering of values must not change while they are stored in the SortedArray.
 *
 * Methods for adding values:
 *
 * - {@link SortedArray#add}
 * - {@link SortedArray#update}
 *
 * Methods for removing values:
 *
 * - {@link SortedArray#clear}
 * - {@link SortedArray#delete}
 * - {@link SortedArray#deleteAt}
 * - {@link SortedArray#deleteSlice}
 * - {@link SortedArray#pop}
 *
 * Methods for looking up values:
 *
 * - {@link SortedArray#at}
 * - {@link SortedArray#bisectLeft}
 * - {@link SortedArray#bisectRight}
 * - {@link SortedArray#count}
 * - {@link SortedArray#includes}
 * - {@link SortedArray#indexOf}
 *
 * Methods for iterating values:
 *
 * - {@link SortedArray#irange}
 * - {@link SortedArray#islice}
 * - {@link SortedArray#slice}
 * - {@link SortedArray#[Symbol.iterator]}
 *
 * Miscellaneous methods and properties:
 *
 * - {@link SortedArray#clone}
 * - {@link SortedArray#concat}
 * - {@link SortedArray#length}
 * - {@link SortedArray#toJSON}
 * - {@link SortedArray#toString}
 *
 * @typeParam T - The element type.
 * @typeParam C - Part of the element that the comparator function sees.
 */
declare class SortedArray<T extends C, C = T> extends AbstractSortedArray<T, C> {
    /**
     * Add `value` to SortedArray.
     *
     * @example
     * const sl = new SortedArray();
     * sl.add(3);
     * sl.add(1);
     * sl.add(2);
     * sl // SortedArray [1, 2, 3]
     *
     * @param value - Value to add to SortedArray.
     */
    add(value: T): void;
    update(iterable: Iterable<T>): void;
    /**
     * Return true if `value` is an element of the SortedArray.
     *
     * @example
     * const sl = new SortedArray([1, 2, 3, 4, 5]);
     * sl.includes(3) // true
     *
     * @param value - Search for value in SortedArray.
     * @returns True if `value` in SortedArray.
     */
    includes(value: C): boolean;
    /**
     * Executes a callback function once for each element in a SortedArray.
     * @param fn - The function to execute.
     * @param thisArg - An object to which the `this` keyword can refer in the callback function.
     * If thisArg is omitted, undefined is used.
     */
    forEach(fn: (value: T, index: number, array: this) => void, thisArg?: any): void;
    /**
     * An alias for {@link values}.
     */
    [Symbol.iterator](): ArrayIterator<T>;
    /**
     * Returns an iterable of integers ranging from 0 (inclusive) to the length of the SortedArray (exclusive).
     */
    keys(): ArrayIterator<number>;
    /**
     * Returns an iterable of values in the SortedArray.
     */
    values(): ArrayIterator<T>;
    /**
     * Returns an iterable of index-value pairs of elements in the SortedArray.
     */
    entries(): ArrayIterator<[number, T]>;
    /**
     * The size of the SortedArray.
     */
    get length(): number;
    /**
     * Return number of occurrences of `value` in the SortedArray.
     *
     * @example
     * const sl = new SortedArray([1, 2, 2, 3, 3, 3, 4, 4, 4, 4]);
     * sl.count(3) // 3
     *
     * @param value - Value to count in SortedArray.
     * @returns Count.
     */
    count(value: T): number;
    /**
     * Return new SortedArray containing all values in both sequences.
     *
     * Values in `other` do not need to be in sorted order.
     *
     * @example
     * const sl1 = new SortedArray('bat');
     * const sl2 = new SortedArray('cat');
     * sl1.concat(sl2) // SortedArray ['a', 'a', 'b', 'c', 't', 't']
     *
     * @param other - Other iterable.
     * @returns New SortedArray.
     */
    concat(other: Iterable<T>): SortedArray<T>;
}
/**
 * Check the invariants of a SortedArray.
 *
 * @throws {Error} If the SortedArray is corrupted.
 */
declare const checkSortedArray: <T extends C, C>(self: SortedArray<T, C>) => void;

/**
 * SortedSet is a sorted mutable set.
 *
 * SortedSet values are maintained in sorted order.
 * The design of SortedSet is simple:
 * SortedSet is implemented as a SortedArray that prevents duplicates to be inserted.
 * It does not use the native Set type at all.
 *
 * SortedSet values must have a total ordering.
 * They are compared using the provided comparator function only;
 * they do not have to be the same object to be considered equal.
 *
 * The total ordering of values must not change while they are stored in the SortedSet.
 *
 * Set methods and properties:
 *
 * - {@link SortedSet#add}
 * - {@link SortedSet#clear}
 * - {@link SortedSet#delete}
 * - {@link SortedSet#forEach}
 * - {@link SortedSet#has}
 * - {@link SortedSet#size}
 * - {@link SortedSet#entries}
 * - {@link SortedSet#keys}
 * - {@link SortedSet#values}
 * - {@link SortedSet#[Symbol.iterator]}
 *
 * SortedArray methods:
 *
 * - {@link SortedSet#at}
 * - {@link SortedSet#bisectLeft}
 * - {@link SortedSet#bisectRight}
 * - {@link SortedSet#deleteAt}
 * - {@link SortedSet#deleteSlice}
 * - {@link SortedSet#indexOf}
 * - {@link SortedSet#irange}
 * - {@link SortedSet#islice}
 * - {@link SortedSet#pop}
 * - {@link SortedSet#reversed}
 *
 * Set-operation methods:
 *
 * - {@link SortedSet#union}
 * - {@link SortedSet#update}
 * - {@link SortedSet#intersection}
 * - {@link SortedSet#intersectionUpdate}
 * - {@link SortedSet#difference}
 * - {@link SortedSet#differenceUpdate}
 * - {@link SortedSet#symmetricDifference}
 * - {@link SortedSet#symmetricDifferenceUpdate}
 * - {@link SortedSet#isSubsetOf}
 * - {@link SortedSet#isSupersetOf}
 * - {@link SortedSet#isDisjointFrom}
 *
 * Miscellaneous methods:
 *
 * - {@link SortedSet#clone}
 * - {@link SortedSet#count}
 * - {@link SortedSet#toString}
 * - {@link SortedSet#toJSON}
 *
 * @typeParam T - The element type.
 * @typeParam C - Part of the element that the comparator function sees.
 */
declare class SortedSet<T extends C, C = T> extends AbstractSortedArray<T, C> {
    /**
     * Return true if `value` is an element of the SortedSet.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.has(3) // true
     *
     * @param value - Search for this value in the SortedSet.
     * @returns true if `value` is in the SortedSet.
     */
    has(value: C): boolean;
    /**
     * Check if all elements of this SortedSet are in the given set.
     *
     * @param other - An iterable.
     * @returns true if all elements in this SortedSet are also in the other set, and false otherwise.
     */
    isSubsetOf(other: Iterable<T>): boolean;
    /**
     * Check if all elements of the given set are in this SortedSet.
     *
     * @param other - An iterable.
     * @returns true if all elements in the other set are also in this SortedSet, and false otherwise.
     */
    isSupersetOf(other: Iterable<C>): boolean;
    /**
     * Check if this SortedSet has no elements in common with the given set.
     *
     * @param other - An iterable.
     * @returns true if this SortedSet has no elements in common with the other set, and false otherwise.
     */
    isDisjointFrom(other: Iterable<T>): boolean;
    /**
     * Return the size of the SortedSet.
     *
     * @returns The size of the SortedSet.
     */
    get size(): number;
    /**
     * Executes a callback function once for each element in a SortedSet.
     *
     * @param fn - The function to execute.
     * @param thisArg - An object to which the `this` keyword can refer in the callback function.
     * If thisArg is omitted, undefined is used.
     */
    forEach(fn: (value: T, value2: T, set: this) => void, thisArg?: any): void;
    /**
     * An alias for {@link values}.
     */
    [Symbol.iterator](): ArrayIterator<T>;
    /**
     * Despite its name, returns an iterable of the values in the SortedSet.
     */
    keys(): SetIterator<T>;
    /**
     * Returns an iterable of values in the SortedSet.
     */
    values(): SetIterator<T>;
    /**
     * Returns an iterable of `[value, value]` pairs for every `value` in the SortedSet.
     */
    entries(): SetIterator<[T, T]>;
    /**
     * Add `value` to SortedSet if it does not already exist.
     *
     * @example
     * const ss = new SortedSet();
     * ss.add(3);
     * ss.add(1);
     * ss.add(2);
     * ss // SortedSet [1, 2, 3]
     *
     * @param value - Value to add to the SortedSet.
     */
    add(value: T): void;
    /**
     * Return number of occurrences of `value` in the SortedSet.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.count(3) // 1
     *
     * @param value - Value to count in the SortedSet.
     * @returns Count.
     */
    count(value: T): 0 | 1;
    /**
     * Return the difference of two sets as a new SortedSet.
     *
     * The difference is all values that are in this SortedSet but not the other iterable.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.difference([4, 5, 6, 7, 5]) // SortedSet [1, 2, 3]
     *
     * @param other - An iterable. It does not have to be a Set-like.
     * @returns The difference as a new SortedSet.
     */
    difference(other: Iterable<T>): SortedSet<T>;
    /**
     * Remove all values of an iterable from this SortedSet.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.differenceUpdate([4, 5, 6, 7, 5]);
     * ss // SortedSet [1, 2, 3]
     *
     * @param other - An iterable.
     */
    differenceUpdate(other: Iterable<T>): void;
    /**
     * Return the intersection of two sets as a new SortedSet.
     *
     * The intersection is all values that are in this sorted set and the other iterable.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.intersection([4, 5, 6, 7, 5]) // SortedSet [4, 5]
     *
     * @param other - An iterable. It does not have to be a Set-like.
     * @returns The intersection as a new SortedSet.
     */
    intersection(other: Iterable<T>): SortedSet<T>;
    /**
     * Update the sorted set with the intersection of two sets.
     *
     * Keep only values found in itself and `other`.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.intersectionUpdate([4, 5, 6, 7, 5]);
     * ss // SortedSet [4, 5]
     *
     * @param other - An iterable.
     */
    intersectionUpdate(other: Iterable<T>): void;
    /**
     * Return the symmetric difference with `other` as a new SortedSet.
     *
     * The symmetric difference is all values tha are in exactly one of the sets.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.symmetricDifference([4, 5, 6, 7, 5]) // SortedSet [1, 2, 3, 6, 7]
     *
     * @param other - An iterable. It does not have to be a Set-like.
     * @returns The symmetric difference as a new SortedSet.
     */
    symmetricDifference(other: Iterable<T>): SortedSet<T>;
    /**
     * Update the sorted set with the symmetric difference with `other`.
     *
     * Keep only values found in exactly one of itself and `other`.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.symmetricDifferenceUpdate([4, 5, 6, 7, 5]);
     * ss // SortedSet [1, 2, 3, 6, 7]
     *
     * @param other - An iterable.
     */
    symmetricDifferenceUpdate(other: Iterable<T>): void;
    /**
     * Return new SortedSet with values from itself and `other`.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.union([4, 5, 6, 7, 5]) // SortedSet [1, 2, 3, 4, 5, 6, 7]
     *
     * @param other - An iterable. It does not have to be a Set-like.
     * @returns The union as a new SortedSet.
     *
     * @see {@link update} for an in-place version of this method.
     */
    union(other: Iterable<T>): SortedSet<T>;
    /**
     * Update the sorted set adding values from `other`.
     *
     * @example
     * const ss = new SortedSet([1, 2, 3, 4, 5]);
     * ss.update([4, 5, 6, 7, 5]);
     * ss // SortedSet [1, 2, 3, 4, 5, 6, 7]
     *
     * @param other - An iterable.
     *
     * @see {@link union} for a similar method that returns a new SortedSet.
     */
    update(other: Iterable<T>): void;
}
/**
 * Check the invariants of a SortedSet.
 *
 * @throws {Error} If the SortedSet is corrupted.
 */
declare function checkSortedSet<T extends C, C>(self: SortedSet<T, C>): void;

/**
 * SortedMap is a sorted mutable mapping.
 *
 * SortedMap keys are maintained in sorted order.
 * The design of SortedMap is simple:
 * SortedMap maintains a SortedArray of key-value pair objects.
 * It does not use the native Map type at all.
 *
 * SortedMap is compatible with Map.
 * A SortedMap can be used wherever a Map is expected, as long as the code relies on duck typing.
 * SortedMap does not actually inherit Map, however.
 *
 * SortedMap keys must have a total ordering.
 * They are compared using the provided comparator function only;
 * they do not have to be the same object to be considered equal.
 * The total ordering of keys must not change while they are stored in the SortedMap.
 *
 * Map methods and properties:
 *
 * - {@link SortedMap#get}
 * - {@link SortedMap#has}
 * - {@link SortedMap#set}
 * - {@link SortedMap#delete}
 * - {@link SortedMap#clear}
 * - {@link SortedMap#forEach}
 * - {@link SortedMap#[Symbol.iterator]}
 * - {@link SortedMap#keys}
 * - {@link SortedMap#values}
 * - {@link SortedMap#entries}
 * - {@link SortedMap#size}
 * - {@link SortedMap#[Symbol.toStringTag]}
 *
 * Additional methods for adding items:
 *
 * - {@link SortedMap#upsert}
 * - {@link SortedMap#update}
 *
 * Additional methods for removing items:
 *
 * - {@link SortedMap#popKey}
 * - {@link SortedMap#popEntry}
 *
 * Additional methods for looking up items:
 *
 * - {@link SortedMap#entryAt}
 *
 * Miscellaneous methods:
 *
 * - {@link SortedMap#clone}
 *
 * SortedArray methods available (applies to keys):
 *
 * - {@link SortedMap#at}
 * - {@link SortedMap#bisectLeft}
 * - {@link SortedMap#bisectRight}
 * - {@link SortedMap#deleteAt}
 * - {@link SortedMap#deleteSlice}
 * - {@link SortedMap#indexOf}
 * - {@link SortedMap#irange}
 * - {@link SortedMap#islice}
 *
 * @typeParam K - The type of keys.
 * @typeParam V - The type of values.
 * @typeParam C - Part of the key that the comparator function sees.
 */
declare class SortedMap<K extends C, V, C = K> extends AbstractSortedArray<K, C> implements Map<K, V> {
    /**
     * Initialize a SortedMap instance, optionally with the given key-value pairs.
     *
     * @example
     * // All of these construct a SortedMap {'alpha' => 1, 'beta' => 2}
     * new SortedMap([['alpha', 1], ['beta', 2]])
     * new SortedMap(Object.entries({alpha: 1, beta: 2}))
     *
     * const m = new Map();
     * m.set('alpha', 1);
     * m.set('beta', 2);
     * new SortedMap(m)
     *
     * @param iterable - Optional iterable argument provides an initial sequence of pairs to initialize the SortedMap.
     * Each pair in the sequence defines the key and corresponding value.
     * If a key is seen more than once, the first key and the last value is stored in the SortedMap.
     * @param options - An object that specifies characteristics about the sorted container.
     */
    constructor(iterable?: Iterable<[K, V]>, options?: SortedArrayConstructorOptions<C>);
    /**
     * @returns The number of entries in the SortedMap.
     */
    get size(): number;
    /**
     * @returns A boolean value indicating whether an entry with the specified key exists or not.
     */
    has(key: C): boolean;
    /**
     * Get the value associated with the `key`, returning undefined if `key` is not found.
     *
     * @param key - The key to look up.
     */
    get(key: C): V | undefined;
    /**
     * Get the value associated with the `key`, with `defaultValue` as a fallback.
     *
     * @param key - The key to look up.
     * @param defaultValue - The value to return if no value is associated with the key.
     */
    get<R>(key: C, defaultValue: R): V | R;
    /**
     * Store an entry in SortedMap with `key` and corresponding `value`.
     * Keep the key but overwrite the value if the key already has an associated value.
     *
     * @example
     * const sd = new SortedMap();
     * sd.set('c', 3);
     * sd.set('a', 1);
     * sd.set('b', 2);
     * sd // SortedMap {'a' => 1, 'b' => 2, 'c' => 3}
     *
     * @param key - Key for entry.
     * @param value - Value for entry.
     */
    set(key: K, value: V): this;
    _expand(pos: number): void;
    /**
     * Remove all items from the SortedMap.
     */
    clear(): void;
    _delete(pos: number, idx: number): void;
    deleteSlice(start?: number, end?: number): void;
    /**
     * An alias for {@link entries}.
     */
    [Symbol.iterator](): MapIterator<[K, V]>;
    /**
     * Return a shallow copy of the SortedMap.
     *
     * @returns A new SortedMap instance.
     */
    clone(): this;
    /**
     * Executes a callback function once for each key-value pair in a SortedMap.
     *
     * @param fn - The function to execute.
     * @param thisArg - An object to which the `this` keyword can refer in the callback function.
     * If thisArg is omitted, undefined is used.
     */
    forEach(fn: (value: V, key: K, map: SortedMap<K, V>) => void, thisArg?: any): void;
    /**
     * Return an iterator of the SortedMap's keys.
     *
     * Iterating a SortedMap while adding or deleting elements may throw an error or silently fail to iterate over all entries.
     * This is different from the native Map.
     */
    keys(): MapIterator<K>;
    /**
     * Return an iterator of key-value pairs over the SortedMap.
     *
     * The returned iterator yields the same object on every iteration to save allocations,
     * so you must unpack the pair before calling `.next()`.
     *
     * Iterating a SortedMap while adding or deleting elements may throw an error or silently fail to iterate over all entries.
     * This is different from the native Map.
     */
    entries(): MapIterator<[K, V]>;
    /**
     * Return an iterator of the SortedMap's values.
     *
     * Iterating a SortedMap while adding or deleting elements may throw an error or silently fail to iterate over all entries.
     * This is different from the native Map.
     */
    values(): MapIterator<V>;
    /**
     * Remove and return value for the item identified by `key`.
     *
     * If the `key` is not found then return undefined.
     *
     * @example
     * const sd = new SortedMap([['a', 1], ['b', 2], ['c', 3]]);
     * sd.popKey('c') // 3
     * sd.popKey('y') // undefined
     *
     * @param key - Key for item.
     * @returns Value for item or undefined.
     */
    popKey(key: C): V | undefined;
    /**
     * Remove and return value for the item identified by `key`.
     *
     * If the `key` is not found then return `defaultValue`.
     *
     * @example
     * const sd = new SortedMap([['a', 1], ['b', 2], ['c', 3]]);
     * sd.popKey('c', 26) // 3
     * sd.popKey('z', 26) // 26
     *
     * @param key - Key for item.
     * @param defaultValue - Default value if key not found.
     * @returns Value for item.
     */
    popKey<R>(key: C, defaultValue: R): V | R;
    /**
     * Remove and return `[key, value]` pair at `index` from the SortedMap.
     *
     * Optional argument `index` defaults to -1, the last item in the SortedMap.
     * Specify index to be 0 for the first item in the sorted dict.
     *
     * If the `index` is out of range, returns undefined.
     *
     * @example
     * const sd = new SortedMap([['a', 1], ['b', 2], ['c', 3]]);
     * sd.popEntry() // ['c', 3]
     * sd.popEntry(0) // ['a', 1]
     * sd.popEntry(100) // undefined
     *
     * @param index - Index of item (default -1).
     * @returns Key and value pair.
     */
    popEntry(index?: number): [K, V] | undefined;
    /**
     * Return `[key, value]` pair at `index` from the SortedMap.
     *
     * Optional argument `index` defaults to -1, the last item in the SortedMap.
     * Specify index to be 0 for the first item in the sorted dict.
     *
     * If the `index` is out of range, returns undefined.
     *
     * @example
     * const sd = new SortedMap([['a', 1], ['b', 2], ['c', 3]]);
     * sd.entryAt() // ['c', 3]
     * sd.entryAt(0) // ['a', 1]
     * sd.entryAt(100) // undefined
     *
     * @param index - Index of item (default -1).
     * @returns Key and value pair.
     */
    entryAt(index?: number): [K, V] | undefined;
    /**
     * Return the value for the entry identified by `key` in the SortedMap.
     *
     * If `key` is in the SortedMap then return its value.
     * If `key` is not in the SortedMap then insert `key` with value `default` and return `default`.
     *
     * @example
     * const sd = new SortedMap();
     * sd.upsert('a', 1) // 1
     * sd.upsert('a', 10) // 1
     * sd // SortedMap {'a' => 1}
     *
     * @param key - Key of the entry.
     * @param defaultValue - Value of the entry if the entry does not exist.
     * @returns Value of the entry after potential insertion.
     */
    upsert(key: K, defaultValue: V): V;
    /**
     * Update SortedMap with entries from `other`.
     *
     * Overwrites existing items.
     *
     * `other` argument may be a Map, a SortedMap, an iterable of pairs.
     * See {@link constructor} for details.
     *
     * @param other - Iterable of pairs.
     */
    update(other: Iterable<[K, V]>): void;
    /**
     * @returns Always `'SortedMap'`.
     */
    get [Symbol.toStringTag](): string;
}
interface SortedMap<K, V, C> {
    /**
     * Return an index to insert `key` in the SortedMap.
     *
     * If the `key` is already present, the insertion point will be the index of that entry.
     *
     * @example
     * const sd = new SortedMap([[10, 4], [11, 2], [12, 6], [13, 2], [14, 4]]);
     * sd.bisectLeft(12) // 2
     *
     * @param key - Key to find the insertion point of.
     * @returns Insertion index of `key` in the SortedMap.
     */
    bisectLeft(key: C): number;
    /**
     * Return an index to insert `key` in the SortedMap.
     *
     * Similar to {@link bisectLeft}, but if `value` is already present, the insertion point will be after (to the right of) the existing entry.
     *
     * @example
     * const sd = new SortedMap([[10, 4], [11, 2], [12, 6], [13, 2], [14, 4]]);
     * sl.bisectRight(12) // 3
     *
     * @param key - Key to find the insertion point of.
     * @returns Insertion index of `key` in the SortedMap.
     */
    bisectRight(key: C): number;
    /**
     * Return the index of `key` in the SortedMap, or -1 if `key` is not present.
     *
     * Index must be between `start` and `end` for the `key` to be considered present.
     * Negative indices count back from the last item.
     *
     * @example
     * const sl = new SortedMap([['a', 1], ['b', 2], ['c', 4]]);
     * sl.indexOf('c') // 2
     * sl.indexOf('z') // -1
     *
     * @param key - Key in SortedMap.
     * @param start - The index at which to start the search (default 0).
     * @param end - The index at which to end the search (default length).
     * @returns The index of the occurrence of `key` in the sorted container, or -1 if it is not present.
     */
    indexOf(key: C, start?: number, end?: number): number;
}
/**
 * Check the invariants of a SortedMap.
 *
 * @throws {Error} If the SortedMap is corrupted.
 */
declare function checkSortedMap<K extends C, V, C>(self: SortedMap<K, V, C>): void;

export { SortedArray, type SortedArrayConstructorOptions, SortedMap, SortedSet, checkSortedArray, checkSortedMap, checkSortedSet };
