// src/bisect.ts
var bisectLeft = (a, x, cmp) => {
  let lo = 0, hi = a.length;
  while (lo < hi) {
    const mid = lo + (hi - lo >>> 1);
    if (cmp(a[mid], x) < 0) {
      lo = mid + 1;
    } else {
      hi = mid;
    }
  }
  return lo;
};
var bisectRight = (a, x, cmp) => {
  let lo = 0, hi = a.length;
  while (lo < hi) {
    const mid = lo + (hi - lo >>> 1);
    if (cmp(x, a[mid]) < 0) {
      hi = mid;
    } else {
      lo = mid + 1;
    }
  }
  return lo;
};
var insort = (a, x, cmp) => {
  a.splice(bisectRight(a, x, cmp), 0, x);
};

// src/common.ts
var defaultComparator = (a, b) => {
  if (a === b) return 0;
  if (a < b) return -1;
  if (a > b) return 1;
  return 0;
};
var IteratorPrototype = Object.getPrototypeOf([][Symbol.iterator]());
function assert(predicate) {
  if (!predicate) throw new Error("assertion failed");
}

// src/abstract-sorted-array.ts
var AbstractSortedArray = class {
  /**
   * The default load factor.
   * Override it with option `loadFactor` at construction time.
   */
  static DEFAULT_LOAD_FACTOR = 1e3;
  /** @internal */
  _load;
  /** @internal */
  _cmp;
  /** @internal */
  _len = 0;
  /** @internal */
  _lists = [];
  /** @internal */
  _maxes = [];
  /** @internal */
  _index = [];
  /** @internal */
  _offset = 0;
  /**
   * Initialize a sorted container instance.
   *
   * Optional `iterable` argument provides an initial iterable of values to initialize the sorted container.
   *
   * @param iterable - Initial values (optional).
   * @param options - An object that specifies characteristics about the sorted container.
   */
  constructor(iterable, options) {
    this._load = options?.loadFactor ?? new.target.DEFAULT_LOAD_FACTOR;
    this._cmp = options?.comparator ?? defaultComparator;
    if (iterable) {
      this.update(iterable);
    }
  }
  /**
   * Remove all values from sorted container.
   */
  clear() {
    this._len = 0;
    this._lists = [];
    this._maxes = [];
    this._index = [];
    this._offset = 0;
  }
  /**
   * Split sublists with length greater than double the load-factor.
   *
   * Updates the index when the sublist length is less than double the load level.
   * This requires incrementing the nodes in a traversal from the leaf node to the root.
   * For an example traversal see {@link _loc}.
   *
   * @internal
   */
  _expand(pos) {
    if (this._lists[pos].length > this._load << 1) {
      const _lists_pos = this._lists[pos];
      const half = _lists_pos.splice(this._load);
      this._maxes[pos] = _lists_pos[_lists_pos.length - 1];
      this._lists.splice(pos + 1, 0, half);
      this._maxes.splice(pos + 1, 0, half[half.length - 1]);
      this._index = [];
    } else {
      if (this._index.length) {
        let child = this._offset + pos;
        while (child > 0) {
          this._index[child]++;
          child = child - 1 >>> 1;
        }
        this._index[0]++;
      }
    }
  }
  /**
   * Initialize from an Array of sorted values.
   *
   * @internal
   */
  _set(values) {
    this._lists = [];
    for (let pos = 0; pos < values.length; pos += this._load) {
      this._lists.push(values.slice(pos, pos + this._load));
    }
    this._maxes = this._lists.map((sublist) => sublist[sublist.length - 1]);
    this._len = values.length;
    this._index = [];
  }
  /**
   * Return true if `value` is an element of the sorted container.
   *
   * @internal
   */
  _has(value) {
    if (!this._maxes.length) return false;
    const pos = bisectLeft(this._maxes, value, this._cmp);
    if (pos === this._maxes.length) return false;
    const idx = bisectLeft(this._lists[pos], value, this._cmp);
    return !this._cmp(this._lists[pos][idx], value);
  }
  /**
   * Remove one `value` from sorted container if it is a member.
   *
   * If `value` is not a member, do nothing.
   *
   * @example
   * const sl = new SortedArray([1, 2, 3, 4, 5])
   * sl.delete(5)
   * sl.delete(0)
   * Array.from(sl) // [1, 2, 3, 4]
   * @param value - Value to discard from SortedArray.
   * @returns Returns true if an element in the SortedArray existed and has been removed, or false if the element does not exist.
   */
  delete(value) {
    if (!this._maxes.length) return false;
    const pos = bisectLeft(this._maxes, value, this._cmp);
    if (pos === this._maxes.length) return false;
    const idx = bisectLeft(this._lists[pos], value, this._cmp);
    if (this._cmp(this._lists[pos][idx], value)) return false;
    this._delete(pos, idx);
    return true;
  }
  /**
   * Delete value at the given `(pos, idx)`.
   *
   * Combines lists that are less than half the load level.
   *
   * Updates the index when the sublist length is more than half the load level.
   * This requires decrementing the nodes in a traversal from the leaf node to the root.
   * For an example traversal see {@link _loc}.
   *
   * @param pos - Lists index.
   * @param idx - Sublist index.
   *
   * @internal
   */
  _delete(pos, idx) {
    const sublist = this._lists[pos];
    sublist.splice(idx, 1);
    this._len--;
    const sublistLength = sublist.length;
    if (sublistLength > this._load >>> 1) {
      this._maxes[pos] = sublist[sublist.length - 1];
      if (this._index.length) {
        for (let child = this._offset + pos; child; child = child - 1 >>> 1) {
          this._index[child]--;
        }
        this._index[0]--;
      }
    } else if (this._lists.length > 1) {
      if (pos === 0) pos++;
      const prev = pos - 1;
      this._lists[prev].push(...this._lists[pos]);
      this._maxes[prev] = this._lists[prev][this._lists[prev].length - 1];
      this._lists.splice(pos, 1);
      this._maxes.splice(pos, 1);
      this._index = [];
      this._expand(prev);
    } else if (sublistLength > 0) {
      this._maxes[pos] = sublist[sublist.length - 1];
    } else {
      this._lists.splice(pos, 1);
      this._maxes.splice(pos, 1);
      this._index = [];
    }
  }
  /**
   * Convert an index pair (lists index, sublist index) into a single index number that corresponds to the position of the value in the sorted container.
   *
   * Many queries require the index be built.
   * Details of the index are described in {@link _build_index}.
   *
   * Indexing requires traversing the tree from a leaf node to the root.
   * The parent of each node is easily computable at `(pos - 1) >>> 1`.
   *
   * Left-child nodes are always at odd indices and right-child nodes are always at even indices.
   *
   * When traversing up from a right-child node, increment the total by the left-child node.
   *
   * The final index is the sum from traversal and the index in the sublist.
   *
   * @param pos - Lists index.
   * @param idx - Sublist index.
   * @returns Index in sorted container.
   *
   * @internal
   */
  _loc(pos, idx) {
    if (pos === 0) return idx;
    if (!this._index.length) this._buildIndex();
    let total = 0;
    pos += this._offset;
    while (pos) {
      if (!(pos & 1)) {
        total += this._index[pos - 1];
      }
      pos = pos - 1 >>> 1;
    }
    return total + idx;
  }
  /**
   * Convert an index into an index pair (lists index, sublist index) that can be used to access the corresponding lists position.
   *
   * Many queries require the index be built.
   * Details of the index are described in {@link _build_index}.
   *
   * Indexing requires traversing the tree to a leaf node.
   * Each node has two children which are easily computable.
   * Given an index, pos, the left-child is at `pos * 2 + 1` and the right-child is at `pos * 2 + 2`.
   *
   * When the index is less than the left-child, traversal moves to the left sub-tree.
   * Otherwise, the index is decremented by the left-child and traversal moves to the right sub-tree.
   *
   * At a child node, the indexing pair is computed from the relative position of the child node as compared with the offset and the remaining index.
   *
   * @param idx - Index in sorted container.
   * @returns (lists index, sublist index) pair.
   *
   * @internal
   */
  _pos(idx) {
    if (idx < 0) {
      const lastLength = this._lists[this._lists.length - 1].length;
      if (-idx <= lastLength) {
        return [this._lists.length - 1, lastLength + idx];
      }
      idx += this._len;
      if (idx < 0) {
        throw new Error("index out of range");
      }
    } else if (idx >= this._len) {
      throw new Error("index out of range");
    }
    if (idx < this._lists[0].length) {
      return [0, idx];
    }
    if (!this._index.length) {
      this._buildIndex();
    }
    let pos = 0;
    let child = 1;
    const indexLength = this._index.length;
    while (child < indexLength) {
      const indexChild = this._index[child];
      if (idx < indexChild) {
        pos = child;
      } else {
        idx -= indexChild;
        pos = child + 1;
      }
      child = (pos << 1) + 1;
    }
    return [pos - this._offset, idx];
  }
  /**
   * Build a positional index for indexing the sorted container.
   *
   * Indexes are represented as binary trees in a dense array notation similar to a binary heap.
   *
   * When built, the index can be used for efficient indexing into the list.
   * See the comment and notes on {@link _pos} for details.
   *
   * @internal
   */
  _buildIndex() {
    const row0 = this._lists.map((list) => list.length);
    const row1 = [];
    for (let i = 1; i < row0.length; i += 2) {
      row1.push(row0[i - 1] + row0[i]);
    }
    if (row0.length & 1) {
      row1.push(row0[row0.length - 1]);
    }
    if (row1.length === 1) {
      this._index = row1.concat(row0);
      this._offset = 1;
      return;
    }
    const size = 1 << 32 - Math.clz32(row1.length - 1);
    row1.push(...Array(size - row1.length).fill(0));
    const tree = [row0, row1];
    while (tree[tree.length - 1].length > 1) {
      const lastLayer = tree[tree.length - 1];
      const row = [];
      for (let i = 1; i < lastLayer.length; i += 2) {
        row.push(lastLayer[i - 1] + lastLayer[i]);
      }
      tree.push(row);
    }
    this._index = tree.reverse().flat();
    this._offset = size * 2 - 1;
  }
  /**
   * Remove and return value at `index` in sorted container.
   *
   * If the sorted container is empty or index is out of range, `undefined` is returned and the sorted container is not modified.
   *
   * Negative indices count back from the last item.
   *
   * @example
   * const sl = new SortedArray('abcde');
   * sl.deleteAt(-1) // 'e'
   * sl.deleteAt(2) // 'c'
   * sl // SortedArray ['a', 'b', 'd']
   *
   * @param index - Index of value. Negative integers count back from the last item in the array.
   * @returns The removed value or `undefined`.
   */
  deleteAt(index) {
    if (!this._len) return;
    if (index === 0) {
      const val2 = this._lists[0][0];
      this._delete(0, 0);
      return val2;
    }
    if (index === -1) {
      const pos2 = this._lists.length - 1;
      const loc = this._lists[pos2].length - 1;
      const val2 = this._lists[pos2][loc];
      this._delete(pos2, loc);
      return val2;
    }
    if (index > 0 && index < this._lists[0].length) {
      const val2 = this._lists[0][index];
      this._delete(0, index);
      return val2;
    }
    const lenLast = this._lists[this._lists.length - 1].length;
    if (index > -lenLast && index < 0) {
      const pos2 = this._lists.length - 1;
      const loc = lenLast + index;
      const val2 = this._lists[pos2][loc];
      this._delete(pos2, loc);
      return val2;
    }
    if (index < -this._len || index >= this._len) return;
    const [pos, idx] = this._pos(index);
    const val = this._lists[pos][idx];
    this._delete(pos, idx);
    return val;
  }
  /**
   * Removes elements from sorted container by index.
   *
   * @example
   * const sl = new SortedArray('abcde');
   * sl.deleteSlice(0, 2);
   * sl // SortedArray ['c', 'd', 'e']
   *
   * @param start The zero-based location in the array from which to start removing elements.
   * Negative integers count back from the last item in the array.
   * @param end The zero-based location in the array at which to stop removing elements.
   * Negative integers count back from the last item in the array.
   * Elements up to but not including `end` are removed.
   */
  deleteSlice(start = 0, end = this._len) {
    if (start < 0) start += this._len;
    start = Math.min(Math.max(start, 0), this._len);
    if (end < 0) end += this._len;
    end = Math.min(Math.max(end, 0), this._len);
    if (start < end) {
      if (start === 0 && end === this._len) {
        return this.clear();
      } else if (this._len <= 8 * (end - start)) {
        let values = this.slice(0, start);
        if (end < this._len) {
          values = values.concat(this.slice(end));
        }
        return this._set(values);
      }
    }
    for (let index = end - 1; index >= start; index--) {
      const [pos, idx] = this._pos(index);
      this._delete(pos, idx);
    }
  }
  /**
   * Returns a copy of a section of sorted container as an ordinary Array.
   *
   * For both start and end, a negative index can be used to indicate an offset from the end of the array.
   *
   * @param start The beginning index of the specified portion of the array.
   * If start is undefined, then the slice begins at index 0.
   * @param end The end index of the specified portion of the array. This is exclusive of the element at the index `end`.
   * If end is undefined, then the slice extends to the end of the array.
   */
  slice(start = 0, end = this._len) {
    if (start < 0) start += this._len;
    start = Math.min(Math.max(start, 0), this._len);
    if (end < 0) end += this._len;
    end = Math.min(Math.max(end, 0), this._len);
    if (start >= end) return [];
    if (start === 0 && end === this._len) {
      return this._lists.flat();
    }
    const [startPos, startIdx] = this._pos(start);
    const startList = this._lists[startPos];
    let endIdx = startIdx + end - start;
    if (startList.length >= endIdx) {
      return startList.slice(startIdx, endIdx);
    }
    let endPos;
    if (end === this._len) {
      endPos = this._lists.length - 1;
      endIdx = this._lists[endPos].length;
    } else {
      [endPos, endIdx] = this._pos(end);
    }
    const parts = this._lists.slice(startPos, endPos);
    parts[0] = parts[0].slice(startIdx);
    parts.push(this._lists[endPos].slice(0, endIdx));
    return parts.flat();
  }
  /**
   * Lookup value at `index` in sorted container.
   *
   * If `index` is out of range, returns `undefined`.
   *
   * @example
   * const sl = new SortedList('abcde');
   * sl.at(1) // 'b'
   * sl.at(-1) // 'e'
   * sl.at(7) // undefined
   *
   * @param index - The zero-based index of the desired code unit. A negative index will count back from the last item.
   * @returns The item located at the specified index, or `undefined` if index out of range.
   */
  at(index) {
    if (!this._len) return;
    if (index === 0) {
      return this._lists[0][0];
    } else if (index === -1) {
      const lastList2 = this._lists[this._lists.length - 1];
      return lastList2[lastList2.length - 1];
    }
    if (index >= 0 && index < this._lists[0].length) {
      return this._lists[0][index];
    }
    const lastList = this._lists[this._lists.length - 1];
    const lastLength = lastList.length;
    if (index > -lastLength && index < 0) {
      return lastList[lastLength + index];
    }
    if (index < -this._len || index >= this._len) return;
    const [pos, idx] = this._pos(index);
    return this._lists[pos][idx];
  }
  /**
   * Return an iterator over the sorted container.
   *
   * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
   */
  _iter() {
    if (!this._len) return [][Symbol.iterator]();
    const maxPos = this._lists.length - 1;
    return this._islice(0, 0, maxPos, this._lists[maxPos].length, false);
  }
  /**
   * Return a reverse iterator over the sorted container.
   *
   * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
   */
  reversed() {
    if (!this._len) return [][Symbol.iterator]();
    const maxPos = this._lists.length - 1;
    return this._islice(0, 0, maxPos, this._lists[maxPos].length, true);
  }
  /**
   * Return an iterator that slices sorted container from `start` to `end`.
   *
   * The `start` and `end` index are treated inclusive and exclusive, respectively.
   *
   * A negative index will count back from the last item.
   *
   * Both `start` and `end` default to `undefined` which is automatically inclusive of the beginning and end of the sorted container.
   *
   * When `reverse` is `true` the values are yielded from the iterator in reverse order; `reverse` defaults to `false`.
   *
   * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
   *
   * @example
   * const sl = new SortedArray('abcdefghij');
   * const it = sl.islice(2, 6);
   * Array.from(it) // ['c', 'd', 'e', 'f']
   *
   * @param start - Start index (inclusive).
   * @param end - Stop index (exclusive).
   * @param reverse - Whether to yield values in reverse order.
   * @returns Iterator.
   */
  islice(start = 0, end = this._len, reverse = false) {
    if (!this._len) return [][Symbol.iterator]();
    if (start < 0) start += this._len;
    start = Math.min(Math.max(start, 0), this._len);
    if (end < 0) end += this._len;
    end = Math.min(Math.max(end, 0), this._len);
    if (start >= end) return [][Symbol.iterator]();
    const [minPos, minIdx] = this._pos(start);
    let maxPos, maxIdx;
    if (end === this._len) {
      maxPos = this._lists.length - 1;
      maxIdx = this._lists[maxPos].length;
    } else {
      [maxPos, maxIdx] = this._pos(end);
    }
    return this._islice(minPos, minIdx, maxPos, maxIdx, reverse);
  }
  /**
   * Return an iterator that slices sorted container using two index pairs.
   *
   * The index pairs are (min_pos, min_idx) and (max_pos, max_idx), the first inclusive and the latter exclusive.
   * See {@link _pos} for details on how an index is converted to an index pair.
   *
   * When `reverse` is `true`, values are yielded from the iterator in reverse order.
   *
   * @internal
   */
  _islice(minPos, minIdx, maxPos, maxIdx, reverse) {
    const lists = this._lists;
    if (reverse) {
      let pos = maxPos;
      let idx = maxIdx;
      return {
        // @ts-expect-error
        __proto__: IteratorPrototype,
        next() {
          if (pos < minPos || pos === minPos && idx <= minIdx) {
            this.value = void 0;
            this.done = true;
            return this;
          }
          idx--;
          if (idx < 0) {
            pos--;
            idx += lists[pos].length;
          }
          this.value = lists[pos][idx];
          return this;
        },
        value: void 0,
        done: false
      };
    } else {
      let pos = minPos;
      let idx = minIdx;
      return {
        // @ts-expect-error
        __proto__: IteratorPrototype,
        next() {
          if (pos > maxPos || pos === maxPos && idx >= maxIdx) {
            this.value = void 0;
            this.done = true;
            return this;
          }
          this.value = lists[pos][idx++];
          if (idx >= lists[pos].length) {
            pos++;
            idx = 0;
          }
          return this;
        },
        value: void 0,
        done: false
      };
    }
  }
  /**
   * Create an iterator of values between `minimum` and `maximum`.
   *
   * Both `minimum` and `maximum` default to `undefined` which is automatically inclusive of the beginning and end of the sorted container.
   *
   * The argument `includeMinimum` and `includeMaximum` is a pair of booleans that indicates whether the minimum and maximum ought to be included in the range, respectively.
   * Both arguments default to `true` such that the range is inclusive of both minimum and maximum.
   *
   * When `reverse` is `true` the values are yielded from the iterator in reverse order; `reverse` defaults to `false`.
   *
   * Iterating a sorted container while adding or deleting elements may throw an error or silently fail to iterate over all elements.
   *
   * @example
   * const sl = new SortedArray('abcdefghij');
   * const it = sl.irange('c', 'f');
   * Array.from(it) // ['c', 'd', 'e', 'f']
   *
   * @param minimum - Minimum value to start iterating.
   * @param maximum - Maximum value to stop iterating.
   * @param includeMinimum - Whether the minimum ought to be included in the range.
   * @param includeMaximum - Whether the maximum ought to be included in the range.
   * @param reverse - Whether to yield values in reverse order.
   * @returns Iterator.
   */
  irange(minimum, maximum, includeMinimum = true, includeMaximum = true, reverse = false) {
    if (!this._maxes.length) return [][Symbol.iterator]();
    let minPos, minIdx;
    if (minimum === void 0) {
      minPos = 0;
      minIdx = 0;
    } else {
      if (includeMinimum) {
        minPos = bisectLeft(this._maxes, minimum, this._cmp);
        if (minPos === this._maxes.length) return [][Symbol.iterator]();
        minIdx = bisectLeft(this._lists[minPos], minimum, this._cmp);
      } else {
        minPos = bisectRight(this._maxes, minimum, this._cmp);
        if (minPos === this._maxes.length) return [][Symbol.iterator]();
        minIdx = bisectRight(this._lists[minPos], minimum, this._cmp);
      }
    }
    let maxPos, maxIdx;
    if (maximum === void 0) {
      maxPos = this._maxes.length - 1;
      maxIdx = this._lists[maxPos].length;
    } else {
      if (includeMaximum) {
        maxPos = bisectRight(this._maxes, maximum, this._cmp);
        if (maxPos === this._maxes.length) {
          maxPos--;
          maxIdx = this._lists[maxPos].length;
        } else {
          maxIdx = bisectRight(this._lists[maxPos], maximum, this._cmp);
        }
      } else {
        maxPos = bisectLeft(this._maxes, maximum, this._cmp);
        if (maxPos === this._maxes.length) {
          maxPos--;
          maxIdx = this._lists[maxPos].length;
        } else {
          maxIdx = bisectLeft(this._lists[maxPos], maximum, this._cmp);
        }
      }
    }
    return this._islice(minPos, minIdx, maxPos, maxIdx, reverse);
  }
  /**
   * Return an index to insert `value` in the sorted container.
   *
   * If the `value` is already present, the insertion point will be before (to the left of) any existing values.
   *
   * @example
   * const sl = new SortedArray([10, 11, 12, 13, 14]);
   * sl.bisectLeft(12) // 2
   *
   * @param value - Value to find the insertion point of.
   * @returns Insertion index of `value` in the sorted container.
   */
  bisectLeft(value) {
    if (!this._maxes.length) return 0;
    const pos = bisectLeft(this._maxes, value, this._cmp);
    if (pos === this._maxes.length) return this._len;
    const idx = bisectLeft(this._lists[pos], value, this._cmp);
    return this._loc(pos, idx);
  }
  /**
   * Return an index to insert `value` in the sorted container.
   *
   * Similar to {@link bisectLeft}, but if `value` is already present, the insertion point will be after (to the right of) any existing values.
   *
   * @example
   * const sl = new SortedArray([10, 11, 12, 13, 14]);
   * sl.bisectRight(12) // 3
   *
   * @param value - Value to find the insertion point of.
   * @returns Insertion index of `value` in the sorted container.
   */
  bisectRight(value) {
    if (!this._maxes.length) return 0;
    const pos = bisectRight(this._maxes, value, this._cmp);
    if (pos === this._maxes.length) return this._len;
    const idx = bisectRight(this._lists[pos], value, this._cmp);
    return this._loc(pos, idx);
  }
  /**
   * Return a shallow copy of the sorted container.
   *
   * @returns A new sorted container.
   */
  clone() {
    return {
      __proto__: Object.getPrototypeOf(this),
      ...this,
      _lists: this._lists.map((sublist) => sublist.slice()),
      _maxes: this._maxes.slice(),
      _index: this._index.slice()
    };
  }
  /**
   * Remove the first element from the sorted container and return it.
   * If the sorted container is empty, undefined is returned and the sorted container is not modified.
   *
   * @example
   * const sl = new SortedArray('abcde');
   * sl.shift() // 'a'
   * sl.shift() // 'b'
   * sl // SortedArray ['c', 'd', 'e']
   *
   * @returns Shifted value or undefined.
   */
  shift() {
    return this.deleteAt(0);
  }
  /**
   * Remove the last element from the sorted container and return it.
   * If the sorted container is empty, undefined is returned and the sorted container is not modified.
   *
   * @example
   * const sl = new SortedArray('abcde');
   * sl.pop() // 'e'
   * sl.pop() // 'd'
   * sl // SortedArray ['a', 'b', 'c']
   *
   * @returns Popped value or undefined.
   */
  pop() {
    return this.deleteAt(-1);
  }
  /**
   * Return first index of `value` in the sorted container, or -1 if `value` is not present.
   *
   * Index must be between `start` and `end` for the `value` to be considered present.
   * Negative indices count back from the last item.
   *
   * @example
   * const sl = new SortedArray('abcde');
   * sl.indexOf('d') // 3
   * sl.indexOf('z') // -1
   *
   * @param value - Value in sorted container.
   * @param start - The array index at which to start the search. If omitted, defaults to 0.
   * @param end - The array index at which to end the search. If omitted, defaults to the end of the sorted container.
   * @returns The index of the first occurrence of `value` in the sorted container, or -1 if it is not present.
   */
  indexOf(value, start = 0, end = this._len) {
    if (!this._len) return -1;
    if (start < 0) start += this._len;
    if (start < 0) start = 0;
    if (end < 0) end += this._len;
    if (end > this._len) end = this._len;
    if (end <= start) return -1;
    const posLeft = bisectLeft(this._maxes, value, this._cmp);
    if (posLeft === this._maxes.length) return -1;
    const idxLeft = bisectLeft(this._lists[posLeft], value, this._cmp);
    if (this._cmp(this._lists[posLeft][idxLeft], value)) return -1;
    const left = this._loc(posLeft, idxLeft);
    if (start <= left) {
      if (left < end) {
        return left;
      }
    } else {
      const right = this.bisectRight(value);
      if (start < right) {
        return start;
      }
    }
    return -1;
  }
  /**
   * Return the first element in the sorted container that is equal to the provided value.
   * If it is not found, undefined is returned.
   *
   * With the default comparator and primitive elements,
   * this method does not tell anything beyond whether the value exists in the sorted container.
   * However, it can turn out to be useful with a custom comparator.
   *
   * @example
   * const sl = new SortedArray(
   *   [
   *     { id: 1, value: 'one' },
   *     { id: 2, value: 'two' },
   *     { id: 3, value: 'three' },
   *   ],
   *   { comparator: (a, b) => a.id - b.id }
   * );
   * sl.find({ id: 2 }) // { id: 2, value: 'two' }
   *
   * @param value - The value to find.
   */
  find(value) {
    if (!this._len) return void 0;
    const pos = bisectLeft(this._maxes, value, this._cmp);
    if (pos === this._maxes.length) return void 0;
    const idx = bisectLeft(this._lists[pos], value, this._cmp);
    if (this._cmp(this._lists[pos][idx], value)) return void 0;
    return this._lists[pos][idx];
  }
  /**
   * Return sorted container as an ordinary Array.
   */
  toJSON() {
    return this._lists.flat();
  }
  /**
   * Return a string representation of sorted container.
   */
  toString() {
    return this._lists.toString();
  }
};
function checkAbstractSortedArray(self) {
  try {
    assert(self._load >= 4);
    assert(self._maxes.length === self._lists.length);
    assert(self._len === self._lists.reduce((acc, list) => acc + list.length, 0));
    for (const sublist of self._lists) {
      for (const val of sublist) {
        assert(self._cmp(val, val) == 0);
      }
    }
    for (const sublist of self._lists) {
      for (let pos = 1; pos < sublist.length; pos++) {
        const le = self._cmp(sublist[pos - 1], sublist[pos]);
        const ge = self._cmp(sublist[pos], sublist[pos - 1]);
        assert(le < 0 && ge > 0 || le === 0 && ge === 0);
      }
    }
    for (let pos = 1; pos < self._lists.length; pos++) {
      const l = self._lists[pos - 1][self._lists[pos - 1].length - 1];
      const r = self._lists[pos][0];
      const le = self._cmp(l, r);
      const ge = self._cmp(r, l);
      assert(le < 0 && ge > 0 || le === 0 && ge === 0);
    }
    for (let pos = 0; pos < self._maxes.length; pos++) {
      assert(Object.is(self._maxes[pos], self._lists[pos][self._lists[pos].length - 1]));
    }
    const double = self._load << 1;
    assert(self._lists.every((list) => list.length <= double));
    const half = self._load >>> 1;
    for (let pos = 0; pos < self._lists.length - 1; pos++) {
      assert(self._lists[pos].length >= half);
    }
    if (self._index.length) {
      assert(self._len === self._index[0]);
      assert(self._index.length === self._offset + self._lists.length);
      for (let pos = 0; pos < self._lists.length; pos++) {
        const leaf = self._index[self._offset + pos];
        assert(leaf === self._lists[pos].length);
      }
      for (let pos = 0; pos < self._offset; pos++) {
        const child = (pos << 1) + 1;
        if (child >= self._index.length) {
          assert(self._index[pos] === 0);
        } else if (child + 1 === self._index.length) {
          assert(self._index[pos] === self._index[child]);
        } else {
          const childSum = self._index[child] + self._index[child + 1];
          assert(childSum === self._index[pos]);
        }
      }
    }
  } catch (e) {
    console.error("len", self._len);
    console.error("load", self._load);
    console.error("offset", self._offset);
    console.error("index", self._index.length, self._index);
    console.error("maxes", self._maxes.length, self._maxes);
    console.error("lists", self._lists.length, self._lists);
    throw e;
  }
}

// src/sorted-array.ts
var SortedArray = class _SortedArray extends AbstractSortedArray {
  /**
   * Add `value` to SortedArray.
   *
   * @example
   * const sl = new SortedArray();
   * sl.add(3);
   * sl.add(1);
   * sl.add(2);
   * sl // SortedArray [1, 2, 3]
   *
   * @param value - Value to add to SortedArray.
   */
  add(value) {
    if (this._maxes.length) {
      let pos = bisectRight(this._maxes, value, this._cmp);
      if (pos === this._maxes.length) {
        pos--;
        this._lists[pos].push(value);
        this._maxes[pos] = value;
      } else {
        insort(this._lists[pos], value, this._cmp);
      }
      this._expand(pos);
    } else {
      this._lists.push([value]);
      this._maxes.push(value);
    }
    this._len++;
  }
  update(iterable) {
    let values = Array.from(iterable).sort(this._cmp);
    if (this._maxes.length) {
      if (values.length * 4 >= this._len) {
        this._lists.push(values);
        values = this._lists.flat();
        values.sort(this._cmp);
        this.clear();
      } else {
        for (const val of values) this.add(val);
        return;
      }
    }
    this._set(values);
  }
  /**
   * Return true if `value` is an element of the SortedArray.
   *
   * @example
   * const sl = new SortedArray([1, 2, 3, 4, 5]);
   * sl.includes(3) // true
   *
   * @param value - Search for value in SortedArray.
   * @returns True if `value` in SortedArray.
   */
  includes(value) {
    return this._has(value);
  }
  /**
   * Executes a callback function once for each element in a SortedArray.
   * @param fn - The function to execute.
   * @param thisArg - An object to which the `this` keyword can refer in the callback function.
   * If thisArg is omitted, undefined is used.
   */
  forEach(fn, thisArg) {
    let idx = 0;
    for (const sublist of this._lists) {
      for (const val of sublist) fn.call(thisArg, val, idx++, this);
    }
  }
  /**
   * An alias for {@link values}.
   */
  [Symbol.iterator]() {
    return this._iter();
  }
  /**
   * Returns an iterable of integers ranging from 0 (inclusive) to the length of the SortedArray (exclusive).
   */
  keys() {
    return Array.prototype.keys.call({ length: this._len });
  }
  /**
   * Returns an iterable of values in the SortedArray.
   */
  values() {
    return this._iter();
  }
  /**
   * Returns an iterable of index-value pairs of elements in the SortedArray.
   */
  *entries() {
    let idx = 0;
    for (const sublist of this._lists) {
      for (const val of sublist) yield [idx++, val];
    }
  }
  /**
   * The size of the SortedArray.
   */
  get length() {
    return this._len;
  }
  /**
   * Return number of occurrences of `value` in the SortedArray.
   *
   * @example
   * const sl = new SortedArray([1, 2, 2, 3, 3, 3, 4, 4, 4, 4]);
   * sl.count(3) // 3
   *
   * @param value - Value to count in SortedArray.
   * @returns Count.
   */
  count(value) {
    if (!this._maxes.length) return 0;
    const posLeft = bisectLeft(this._maxes, value, this._cmp);
    if (posLeft === this._maxes.length) return 0;
    const idxLeft = bisectLeft(this._lists[posLeft], value, this._cmp);
    const posRight = bisectRight(this._maxes, value, this._cmp);
    if (posRight === this._maxes.length) {
      return this._len - this._loc(posLeft, idxLeft);
    }
    const idxRight = bisectRight(this._lists[posRight], value, this._cmp);
    if (posLeft === posRight) {
      return idxRight - idxLeft;
    }
    const right = this._loc(posRight, idxRight);
    const left = this._loc(posLeft, idxLeft);
    return right - left;
  }
  /**
   * Return new SortedArray containing all values in both sequences.
   *
   * Values in `other` do not need to be in sorted order.
   *
   * @example
   * const sl1 = new SortedArray('bat');
   * const sl2 = new SortedArray('cat');
   * sl1.concat(sl2) // SortedArray ['a', 'a', 'b', 'c', 't', 't']
   *
   * @param other - Other iterable.
   * @returns New SortedArray.
   */
  concat(other) {
    return new _SortedArray([...this._lists.flat(), ...other], { comparator: this._cmp });
  }
};
var checkSortedArray = checkAbstractSortedArray;

// src/sorted-set.ts
function iterableToSortedArraySet(iterable, cmp) {
  const arr = Array.from(iterable);
  if (!arr.length) return arr;
  arr.sort(cmp);
  const deduplicated = [arr[0]];
  for (let i = 1; i < arr.length; i++) {
    if (cmp(arr[i - 1], arr[i])) {
      deduplicated.push(arr[i]);
    }
  }
  return deduplicated;
}
var SortedSet = class extends AbstractSortedArray {
  /**
   * Return true if `value` is an element of the SortedSet.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.has(3) // true
   *
   * @param value - Search for this value in the SortedSet.
   * @returns true if `value` is in the SortedSet.
   */
  has(value) {
    return this._has(value);
  }
  /**
   * Check if all elements of this SortedSet are in the given set.
   *
   * @param other - An iterable.
   * @returns true if all elements in this SortedSet are also in the other set, and false otherwise.
   */
  isSubsetOf(other) {
    const arr = Array.from(other).sort(this._cmp);
    for (const sublist of this._lists) {
      for (const val of sublist) {
        const idx = bisectLeft(arr, val, this._cmp);
        if (idx >= arr.length || this._cmp(arr[idx], val)) return false;
      }
    }
    return true;
  }
  /**
   * Check if all elements of the given set are in this SortedSet.
   *
   * @param other - An iterable.
   * @returns true if all elements in the other set are also in this SortedSet, and false otherwise.
   */
  isSupersetOf(other) {
    for (const val of other) {
      if (!this.has(val)) return false;
    }
    return true;
  }
  /**
   * Check if this SortedSet has no elements in common with the given set.
   *
   * @param other - An iterable.
   * @returns true if this SortedSet has no elements in common with the other set, and false otherwise.
   */
  isDisjointFrom(other) {
    for (const val of other) {
      if (this.has(val)) return false;
    }
    return true;
  }
  /**
   * Return the size of the SortedSet.
   *
   * @returns The size of the SortedSet.
   */
  get size() {
    return this._len;
  }
  /**
   * Executes a callback function once for each element in a SortedSet.
   *
   * @param fn - The function to execute.
   * @param thisArg - An object to which the `this` keyword can refer in the callback function.
   * If thisArg is omitted, undefined is used.
   */
  forEach(fn, thisArg) {
    for (const sublist of this._lists) {
      for (const val of sublist) fn.call(thisArg, val, val, this);
    }
  }
  /**
   * An alias for {@link values}.
   */
  [Symbol.iterator]() {
    return this._iter();
  }
  /**
   * Despite its name, returns an iterable of the values in the SortedSet.
   */
  keys() {
    return this._iter();
  }
  /**
   * Returns an iterable of values in the SortedSet.
   */
  values() {
    return this._iter();
  }
  /**
   * Returns an iterable of `[value, value]` pairs for every `value` in the SortedSet.
   */
  *entries() {
    for (const sublist of this._lists) {
      for (const val of sublist) yield [val, val];
    }
  }
  /**
   * Add `value` to SortedSet if it does not already exist.
   *
   * @example
   * const ss = new SortedSet();
   * ss.add(3);
   * ss.add(1);
   * ss.add(2);
   * ss // SortedSet [1, 2, 3]
   *
   * @param value - Value to add to the SortedSet.
   */
  add(value) {
    if (this._len) {
      let pos = bisectLeft(this._maxes, value, this._cmp);
      if (pos === this._maxes.length) {
        pos--;
        this._lists[pos].push(value);
        this._maxes[pos] = value;
      } else {
        const sublist = this._lists[pos];
        const idx = bisectLeft(sublist, value, this._cmp);
        if (!this._cmp(sublist[idx], value)) return;
        sublist.splice(idx, 0, value);
      }
      this._expand(pos);
    } else {
      this._lists.push([value]);
      this._maxes.push(value);
    }
    this._len++;
  }
  /**
   * Return number of occurrences of `value` in the SortedSet.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.count(3) // 1
   *
   * @param value - Value to count in the SortedSet.
   * @returns Count.
   */
  count(value) {
    return +this.has(value);
  }
  /**
   * Return the difference of two sets as a new SortedSet.
   *
   * The difference is all values that are in this SortedSet but not the other iterable.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.difference([4, 5, 6, 7, 5]) // SortedSet [1, 2, 3]
   *
   * @param other - An iterable. It does not have to be a Set-like.
   * @returns The difference as a new SortedSet.
   */
  difference(other) {
    const ret = this.clone();
    ret.differenceUpdate(other);
    return ret;
  }
  /**
   * Remove all values of an iterable from this SortedSet.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.differenceUpdate([4, 5, 6, 7, 5]);
   * ss // SortedSet [1, 2, 3]
   *
   * @param other - An iterable.
   */
  differenceUpdate(other) {
    const values = iterableToSortedArraySet(other, this._cmp);
    if (values.length * 4 < this._len) {
      for (const value of values) {
        this.delete(value);
      }
      return;
    }
    const result = [];
    for (const sublist of this._lists) {
      for (const value of sublist) {
        const idx = bisectLeft(values, value, this._cmp);
        if (idx >= values.length || this._cmp(values[idx], value)) {
          result.push(value);
        }
      }
    }
    this._set(result);
  }
  /**
   * Return the intersection of two sets as a new SortedSet.
   *
   * The intersection is all values that are in this sorted set and the other iterable.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.intersection([4, 5, 6, 7, 5]) // SortedSet [4, 5]
   *
   * @param other - An iterable. It does not have to be a Set-like.
   * @returns The intersection as a new SortedSet.
   */
  intersection(other) {
    const ret = this.clone();
    ret.intersectionUpdate(other);
    return ret;
  }
  /**
   * Update the sorted set with the intersection of two sets.
   *
   * Keep only values found in itself and `other`.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.intersectionUpdate([4, 5, 6, 7, 5]);
   * ss // SortedSet [4, 5]
   *
   * @param other - An iterable.
   */
  intersectionUpdate(other) {
    const values = iterableToSortedArraySet(other, this._cmp);
    const result = [];
    if (this._len < values.length) {
      for (const sublist of this._lists) {
        for (const value of sublist) {
          const idx = bisectLeft(values, value, this._cmp);
          if (idx < values.length && !this._cmp(values[idx], value)) {
            result.push(value);
          }
        }
      }
    } else {
      for (const value of values) {
        if (value === void 0) {
          const index = this.indexOf(value);
          if (index >= 0) {
            result.push(this.at(index));
          }
        } else {
          const thisValue = this.find(value);
          if (thisValue) {
            result.push(thisValue);
          }
        }
      }
    }
    this._set(result);
  }
  /**
   * Return the symmetric difference with `other` as a new SortedSet.
   *
   * The symmetric difference is all values tha are in exactly one of the sets.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.symmetricDifference([4, 5, 6, 7, 5]) // SortedSet [1, 2, 3, 6, 7]
   *
   * @param other - An iterable. It does not have to be a Set-like.
   * @returns The symmetric difference as a new SortedSet.
   */
  symmetricDifference(other) {
    const ret = this.clone();
    ret.symmetricDifferenceUpdate(other);
    return ret;
  }
  /**
   * Update the sorted set with the symmetric difference with `other`.
   *
   * Keep only values found in exactly one of itself and `other`.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.symmetricDifferenceUpdate([4, 5, 6, 7, 5]);
   * ss // SortedSet [1, 2, 3, 6, 7]
   *
   * @param other - An iterable.
   */
  symmetricDifferenceUpdate(other) {
    const result = this._lists.flat();
    for (const value of other) {
      const index = this.indexOf(value);
      if (index >= 0) {
        delete result[index];
      } else {
        result.push(value);
      }
    }
    this.clear();
    this.update(result.filter((_, i, a) => i in a));
  }
  /**
   * Return new SortedSet with values from itself and `other`.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.union([4, 5, 6, 7, 5]) // SortedSet [1, 2, 3, 4, 5, 6, 7]
   *
   * @param other - An iterable. It does not have to be a Set-like.
   * @returns The union as a new SortedSet.
   *
   * @see {@link update} for an in-place version of this method.
   */
  union(other) {
    const ret = this.clone();
    ret.update(other);
    return ret;
  }
  /**
   * Update the sorted set adding values from `other`.
   *
   * @example
   * const ss = new SortedSet([1, 2, 3, 4, 5]);
   * ss.update([4, 5, 6, 7, 5]);
   * ss // SortedSet [1, 2, 3, 4, 5, 6, 7]
   *
   * @param other - An iterable.
   *
   * @see {@link union} for a similar method that returns a new SortedSet.
   */
  update(other) {
    if (this._len) {
      const values = Array.from(other);
      if (values.length * 4 >= this._len) {
        this._lists.push(values);
        this._set(iterableToSortedArraySet(this._lists.flat(), this._cmp));
      } else {
        for (const val of other) this.add(val);
        return;
      }
    } else {
      this._set(iterableToSortedArraySet(other, this._cmp));
    }
  }
};
function checkSortedSet(self) {
  checkAbstractSortedArray(self);
  for (const sublist of self._lists) {
    for (let pos = 1; pos < sublist.length; pos++) {
      assert(self._cmp(sublist[pos - 1], sublist[pos]) < 0);
    }
  }
  for (let pos = 1; pos < self._lists.length; pos++) {
    const l = self._lists[pos - 1][self._lists[pos - 1].length - 1];
    const r = self._lists[pos][0];
    assert(self._cmp(l, r) < 0);
  }
}

// src/sorted-map.ts
var SortedMap = class extends AbstractSortedArray {
  /** @internal */
  _values = [];
  /**
   * Initialize a SortedMap instance, optionally with the given key-value pairs.
   *
   * @example
   * // All of these construct a SortedMap {'alpha' => 1, 'beta' => 2}
   * new SortedMap([['alpha', 1], ['beta', 2]])
   * new SortedMap(Object.entries({alpha: 1, beta: 2}))
   *
   * const m = new Map();
   * m.set('alpha', 1);
   * m.set('beta', 2);
   * new SortedMap(m)
   *
   * @param iterable - Optional iterable argument provides an initial sequence of pairs to initialize the SortedMap.
   * Each pair in the sequence defines the key and corresponding value.
   * If a key is seen more than once, the first key and the last value is stored in the SortedMap.
   * @param options - An object that specifies characteristics about the sorted container.
   */
  constructor(iterable, options) {
    super(void 0, options);
    if (iterable) {
      this.update(iterable);
    }
  }
  /**
   * @returns The number of entries in the SortedMap.
   */
  get size() {
    return this._len;
  }
  /**
   * @returns A boolean value indicating whether an entry with the specified key exists or not.
   */
  has(key) {
    return super._has(key);
  }
  get(key, defaultValue) {
    const pos = bisectLeft(this._maxes, key, this._cmp);
    if (pos === this._maxes.length) return defaultValue;
    const idx = bisectLeft(this._lists[pos], key, this._cmp);
    if (this._cmp(this._lists[pos][idx], key)) return defaultValue;
    return this._values[pos][idx];
  }
  /**
   * Store an entry in SortedMap with `key` and corresponding `value`.
   * Keep the key but overwrite the value if the key already has an associated value.
   *
   * @example
   * const sd = new SortedMap();
   * sd.set('c', 3);
   * sd.set('a', 1);
   * sd.set('b', 2);
   * sd // SortedMap {'a' => 1, 'b' => 2, 'c' => 3}
   *
   * @param key - Key for entry.
   * @param value - Value for entry.
   */
  set(key, value) {
    if (this._len) {
      let pos = bisectLeft(this._maxes, key, this._cmp);
      if (pos === this._maxes.length) {
        pos--;
        this._lists[pos].push(key);
        this._values[pos].push(value);
        this._maxes[pos] = key;
      } else {
        const sublist = this._lists[pos];
        const valList = this._values[pos];
        const idx = bisectLeft(sublist, key, this._cmp);
        const val = sublist[idx];
        if (!this._cmp(val, key)) {
          valList[idx] = value;
          return this;
        }
        sublist.splice(idx, 0, key);
        valList.splice(idx, 0, value);
      }
      this._expand(pos);
    } else {
      this._lists.push([key]);
      this._values.push([value]);
      this._maxes.push(key);
    }
    this._len++;
    return this;
  }
  _expand(pos) {
    super._expand(pos);
    const sublist = this._values[pos];
    if (sublist.length > this._load << 1) {
      const half = sublist.splice(this._load);
      this._values.splice(pos + 1, 0, half);
    }
  }
  /**
   * Remove all items from the SortedMap.
   */
  clear() {
    super.clear();
    this._values = [];
  }
  _delete(pos, idx) {
    const sublist = this._values[pos];
    sublist.splice(idx, 1);
    const sublistLength = sublist.length;
    if (sublistLength <= this._load >>> 1) {
      if (this._values.length > 1) {
        const prev = pos && pos - 1;
        this._values[prev].push(...this._values[prev + 1]);
        this._values.splice(prev + 1, 1);
      } else if (!sublistLength) {
        this._values.pop();
      }
    }
    super._delete(pos, idx);
  }
  deleteSlice(start = 0, end = this._len) {
    if (start < 0) start += this._len;
    start = Math.min(Math.max(start, 0), this._len);
    if (end < 0) end += this._len;
    end = Math.min(Math.max(end, 0), this._len);
    if (start >= end) return;
    if (start === 0 && end === this._len) {
      return this.clear();
    }
    for (let index = end - 1; index >= start; index--) {
      const [pos, idx] = this._pos(index);
      this._delete(pos, idx);
    }
  }
  /**
   * An alias for {@link entries}.
   */
  [Symbol.iterator]() {
    return this.entries();
  }
  /**
   * Return a shallow copy of the SortedMap.
   *
   * @returns A new SortedMap instance.
   */
  clone() {
    const that = super.clone();
    that._values = this._values.map((sublist) => sublist.slice());
    return that;
  }
  /**
   * Executes a callback function once for each key-value pair in a SortedMap.
   *
   * @param fn - The function to execute.
   * @param thisArg - An object to which the `this` keyword can refer in the callback function.
   * If thisArg is omitted, undefined is used.
   */
  forEach(fn, thisArg) {
    for (let pos = 0; pos < this._lists.length; pos++) {
      const sublist = this._lists[pos];
      const valList = this._values[pos];
      for (let idx = 0; idx < sublist.length; idx++) {
        fn.call(thisArg, valList[idx], sublist[idx], this);
      }
    }
  }
  /**
   * Return an iterator of the SortedMap's keys.
   *
   * Iterating a SortedMap while adding or deleting elements may throw an error or silently fail to iterate over all entries.
   * This is different from the native Map.
   */
  keys() {
    return super._iter();
  }
  /**
   * Return an iterator of key-value pairs over the SortedMap.
   *
   * The returned iterator yields the same object on every iteration to save allocations,
   * so you must unpack the pair before calling `.next()`.
   *
   * Iterating a SortedMap while adding or deleting elements may throw an error or silently fail to iterate over all entries.
   * This is different from the native Map.
   */
  entries() {
    const lists = this._lists;
    const values = this._values;
    let pos = 0;
    let idx = 0;
    return {
      // @ts-expect-error
      __proto__: IteratorPrototype,
      next() {
        if (pos >= lists.length) {
          this.value = void 0;
          this.done = true;
        } else {
          this.value = [lists[pos][idx], values[pos][idx]];
          idx++;
          if (idx >= lists[pos].length) {
            pos++;
            idx = 0;
          }
        }
        return this;
      },
      value: void 0,
      done: false
    };
  }
  /**
   * Return an iterator of the SortedMap's values.
   *
   * Iterating a SortedMap while adding or deleting elements may throw an error or silently fail to iterate over all entries.
   * This is different from the native Map.
   */
  values() {
    const values = this._values;
    let pos = 0;
    let idx = 0;
    return {
      // @ts-expect-error
      __proto__: IteratorPrototype,
      next() {
        if (pos >= values.length) {
          this.value = void 0;
          this.done = true;
        } else {
          this.value = values[pos][idx++];
          if (idx >= values[pos].length) {
            pos++;
            idx = 0;
          }
        }
        return this;
      },
      value: void 0,
      done: false
    };
  }
  popKey(key, defaultValue) {
    const val = this.get(key, defaultValue);
    this.delete(key);
    return val;
  }
  /**
   * Remove and return `[key, value]` pair at `index` from the SortedMap.
   *
   * Optional argument `index` defaults to -1, the last item in the SortedMap.
   * Specify index to be 0 for the first item in the sorted dict.
   *
   * If the `index` is out of range, returns undefined.
   *
   * @example
   * const sd = new SortedMap([['a', 1], ['b', 2], ['c', 3]]);
   * sd.popEntry() // ['c', 3]
   * sd.popEntry(0) // ['a', 1]
   * sd.popEntry(100) // undefined
   *
   * @param index - Index of item (default -1).
   * @returns Key and value pair.
   */
  popEntry(index = -1) {
    const entry = this.entryAt(index);
    this.deleteAt(index);
    return entry;
  }
  /**
   * Return `[key, value]` pair at `index` from the SortedMap.
   *
   * Optional argument `index` defaults to -1, the last item in the SortedMap.
   * Specify index to be 0 for the first item in the sorted dict.
   *
   * If the `index` is out of range, returns undefined.
   *
   * @example
   * const sd = new SortedMap([['a', 1], ['b', 2], ['c', 3]]);
   * sd.entryAt() // ['c', 3]
   * sd.entryAt(0) // ['a', 1]
   * sd.entryAt(100) // undefined
   *
   * @param index - Index of item (default -1).
   * @returns Key and value pair.
   */
  entryAt(index = -1) {
    if (index < -this._len || index >= this._len) return void 0;
    const [pos, idx] = this._pos(index);
    return [this._lists[pos][idx], this._values[pos][idx]];
  }
  /**
   * Return the value for the entry identified by `key` in the SortedMap.
   *
   * If `key` is in the SortedMap then return its value.
   * If `key` is not in the SortedMap then insert `key` with value `default` and return `default`.
   *
   * @example
   * const sd = new SortedMap();
   * sd.upsert('a', 1) // 1
   * sd.upsert('a', 10) // 1
   * sd // SortedMap {'a' => 1}
   *
   * @param key - Key of the entry.
   * @param defaultValue - Value of the entry if the entry does not exist.
   * @returns Value of the entry after potential insertion.
   */
  upsert(key, defaultValue) {
    if (this.has(key)) {
      return this.get(key);
    }
    this.set(key, defaultValue);
    return defaultValue;
  }
  /**
   * Update SortedMap with entries from `other`.
   *
   * Overwrites existing items.
   *
   * `other` argument may be a Map, a SortedMap, an iterable of pairs.
   * See {@link constructor} for details.
   *
   * @param other - Iterable of pairs.
   */
  update(other) {
    for (const [key, value] of Array.from(other)) {
      this.set(key, value);
    }
  }
  /**
   * @returns Always `'SortedMap'`.
   */
  get [Symbol.toStringTag]() {
    return "SortedMap";
  }
};
function checkSortedMap(self) {
  checkAbstractSortedArray(self);
  assert(self._lists.length === self._values.length);
  for (let pos = 0; pos < self._lists.length; pos++) {
    assert(self._lists[pos].length === self._values[pos].length);
  }
}
export {
  SortedArray,
  SortedMap,
  SortedSet,
  checkSortedArray,
  checkSortedMap,
  checkSortedSet
};
